Package: SMTrackR
Type: Package
Title: SMTrackR: an R/Bioconductor package for mapping protein binding
        at individual DNA molecules
Version: 1.1.0
Date: 2024-12-31
Authors@R: 
    c( person (given = "Aashna",
               family = "Bansal", 
               role = c("aut", "ctb"),
               email = "aashna_b@bt.iitr.ac.in"),
       person (given = "Himani",
               family = "Barmola", 
               role = c("aut", "ctb"),
               email = "km_h@bt.iitr.ac.in"),
       person (given = "Shivam",
               family = "Yadav", 
               role = c("aut", "ctb"),
               email = "shivam_y@bt.iitr.ac.in"),
       person (given = "Satyanarayan",
               family = "Rao", 
               role = c("aut", "cre"),
               email = "satyanarayan.rao@bt.iitr.ac.in",
               comment = c(ORCID="0000-0001-7678-3505"))
     )
Description: The package uses exogenous enzyme imprinted information to map protein-DNA binding on individual sequenced DNA molecules. For example, GpC methyltransferase, CpG methyltransferase, and Adenine methyltransferases. Public datasets from such assays are compiled into tracks, and hosted at public servers like Galaxy for their seamless access by this package.
biocViews: NucleosomePositioning, Visualization, GeneTarget,
        GenomeAssembly
Imports: jsonlite, GenomicRanges, rtracklayer, stringr, BiocFileCache,
        S4Vectors
License: MIT + file LICENSE
Suggests: knitr, rmarkdown, BiocStyle
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.2
Depends: R (>= 4.5)
URL: https://www.raolab.in
BugReports: https://github.com/satyanarayan-rao/SMTrackR
Contact: mailto:satyanarayan.rao@bt.iitr.ac.in
git_url: https://git.bioconductor.org/packages/SMTrackR
git_branch: devel
git_last_commit: bff9c40
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: no
Packaged: 2026-05-18 23:29:29 UTC; biocbuild
Author: Aashna Bansal [aut, ctb],
  Himani Barmola [aut, ctb],
  Shivam Yadav [aut, ctb],
  Satyanarayan Rao [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7678-3505>)
Maintainer: Satyanarayan Rao <satyanarayan.rao@bt.iitr.ac.in>
Built: R 4.6.0; ; 2026-05-19 05:10:11 UTC; unix
