exportPattern("plotFootprints")
exportPattern("plotFootprintsUsingLocalBigBed")
exportPattern("plotMethylationCallsNanopore")
exportPattern("plotMethylationCallsNanoporeUsingLocalBigBed")
exportPattern("generateGvizCodeforSMF")
exportPattern("listTracks")


importFrom("jsonlite", "fromJSON")
importFrom("rtracklayer", "import")
importFrom("GenomicRanges", "resize")
importFrom("GenomicRanges", "findOverlaps")
importFrom("GenomicRanges", "width")
importFrom("S4Vectors", "from")
importFrom("S4Vectors", "to")
importFrom("stringr", "str_split")
importFrom("stringr", "str_length")
importFrom("stringr", "str_sub")
importFrom("stringr", "str_locate_all")
importFrom("stringr", "str_count")
importFrom("stringr", "str_replace_all")
importFrom("grDevices", "dev.copy", "dev.off", "pdf", "png",
           "postscript", "dev.control")
importFrom("utils", "head")
importFrom("graphics", "axis", "box", "image", "par", "segments",
           "text")
importFrom("utils", "download.file", "read.table", "write.table")
importFrom("BiocFileCache", "bfcrpath")
