## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
options(warn=-1)

## ----echo=FALSE, out.width="70%", fig.align="center"--------------------------
knitr::include_graphics("images/schematic_for_vignettes.png")

## ----echo = TRUE--------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("SMTrackR")
# rmarkdown::html_vignette

## ----Load package, echo=TRUE--------------------------------------------------
library(SMTrackR)

## ----plot psuedo-bulk scNOMe-seq data, fig.width=4.5, fig.height=6, echo=TRUE----
SMTrackR::plotFootprints(organism = "mmusculus", model = "16cell", condition = "WT",
                         genome_assembly = "mm39", type = "SMF", chromosome = "chr1",
                         start = 191450362,  end = 191450392, tr = "16cell",
                         label = "tss_mm39", fp_cap = 50, remove_dup = FALSE, target_dir = "smf")
SMTrackR::generateGvizCodeforSMF(organism = "mmusculus", model = "16cell", condition = "WT",
                         genome_assembly = "mm39", type = "SMF", chromosome = "chr1",
                         start = 191450362,  end = 191450392, tr = "16cell",
                         label = "tss_mm39", fp_cap = 50, remove_dup = FALSE, gviz_left = 500,
                         gviz_right = 500, target_dir = "smf")

## ----echo=FALSE, out.width="100%", fig.align="center"-------------------------
knitr::include_graphics("images/tss_mm39.gviz.png")

## ----Plot dSMF data, fig.width=4.5, fig.height=6, echo=TRUE-------------------
SMTrackR::plotFootprints(target_dir = "dsmf")

## ----echo=FALSE, out.width="100%", fig.align="center"-------------------------
knitr::include_graphics("images/peak229.heatmap.png")

## ----Plot Methylation Calls on Nanopore Reads, fig.width=10, fig.height=5, echo=TRUE----
SMTrackR::plotMethylationCallsNanopore(target_dir = "nanopore") 

## ----echo=FALSE, out.width="100%", fig.align="center"-------------------------
knitr::include_graphics("images/smac_seq.heatmap.png")

## ----List available tracks, echo = TRUE---------------------------------------

#SMTrackR::listTracks()

## ----Session Info, echo = TRUE------------------------------------------------
sessionInfo()

