% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/method.R
\name{listTracks}
\alias{listTracks}
\title{Discover Available Single Molecule Data Tracks}
\usage{
listTracks()
}
\value{
A tibble containing track metadata with columns:
\itemize{
  \item organism: Species identifier (e.g. "dmelanogaster")
  \item genome_assembly: Reference genome version
  \item condition: Experimental condition
  \item model: Biological model/system
  \item track_name: Identifier for `tr` parameter in `plotFootprints`
  \item data_type: NOME-seq protocol variant
  \item bigbed_url: Cloud-hosted data location
}
}
\description{
Retrieves track metadata from a centralized Google Sheet containing
organism-specific single-molecule datasets and experimental conditions.
}
\examples{
# List all available tracks from default repository
listTracks()


}
\seealso{
`plotFootprints()`: Use track names from this list to visualize specific datasets
}
