% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/method.R
\name{plotFootprintsUsingLocalBigBed}
\alias{plotFootprintsUsingLocalBigBed}
\title{Visualize Single Molecule Footprint Patterns using Local BigBed file}
\usage{
plotFootprintsUsingLocalBigBed(
  bigBed = "",
  organism = "dmelanogaster",
  model = "S2",
  condition = "WT",
  genome_assembly = "dm3",
  chromosome = "chr2L",
  start = 480290,
  end = 480320,
  label = "peak229",
  span_left = 150,
  span_right = 150,
  remove_dup = FALSE,
  fp_cap = 50,
  target_dir = ""
)
}
\arguments{
\item{bigBed}{BigBed file path (e.g., "demo.bb")}

\item{organism}{Organism code (default: "dmelanogaster")}

\item{model}{Biological model/system (default: "S2" cells)}

\item{condition}{Experimental condition (default: "WT")}

\item{genome_assembly}{Genome version (default: "dm6")}

\item{chromosome}{Chromosome ID (e.g., "chr2L")}

\item{start}{Genomic start position (numeric/character)}

\item{end}{Genomic end position (numeric/character)}

\item{label}{Plot title annotation}

\item{span_left}{Upstream window size from region (default: 150)}

\item{span_right}{Downstream window size from region (default: 150)}

\item{remove_dup}{Remove duplicate reads? (default: FALSE)}

\item{fp_cap}{Maximum footprint value for y-axis scaling (default: 50)}

\item{target_dir}{destination directory where outputs will be stored}
}
\value{
Saves a heatmap in pdf, png, and eps file formats. Also, occupancies are saved in a tsv file
}
\description{
Generates footprint plots from SMF/dSMF data within specified genomic regions.
Designed for Drosophila melanogaster analysis with expandable parameters for other organisms.
}
\details{
This function uses local SMF/dSMF bigBed file and plot heatmap
}
\examples{
# Basic usage with default parameters
plotFootprintsUsingLocalBigBed()

# Custom genomic region analysis
plotFootprintsUsingLocalBigBed(
  bigBed = "inst/extdata/demo.bb", organism = "dmelanogaster",
  model = "S2", condition = "WT", genome_assembly = "dm3",
  chromosome = "chr2L", start = 480290, end = 480320, label = "peak229")

}
